--[[
    CraftDB keeps track of crafters on your server. Whenever you open a
    tradeskill link, the skills and the name of the crafter will be saved
    and can be displayed when needed.
    Copyright (C) 2010  gorilivan@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
]]

local CraftDB = CreateFrame("Frame", "CraftDB_Frame", UIParent)
CraftDB:Hide()

local db = nil
local spellDB = nil
local faction = UnitFactionGroup("player")
local serverfaction = GetRealmName() .. faction
local tradeIDs = {
	2259, -- Alchemy
	2018, -- Blacksmithing
	7411, -- Enchanting
	4036, -- Engineering
	45357, -- Inscription
	25229, -- Jewelcrafting
	2108, -- Leatherworking
	3908, -- Tailoring
	2550, -- Cooking
	3273, -- First Aid
}
local display = {}
local displayTrade = 179
local DataBroker = LibStub:GetLibrary("LibDataBroker-1.1",true)

CraftDB:SetScript("OnEvent", function(self, event, ...)
	self[event](CraftDB, CraftDB, ...)
end
)
CraftDB:RegisterEvent("ADDON_LOADED")
CraftDB:RegisterEvent("TRADE_SKILL_SHOW")

function CraftDB:TRADE_SKILL_SHOW(self)
	--self:print(gsub(GetTradeSkillListLink(), "\124", "\124\124"))
	--|cffffd000|Htrade:27028:375:375:3111EE:xG{_yK|h[First Aid]|h|r
	local tradeLink = GetTradeSkillListLink()
	if not tradeLink then
		return
	end
	local tradeID = self:GetRealTradeID(gsub(tradeLink, "^|c%x+|Htrade:(%d+):.*", "%1"))
	if not tradeID then
		return
	end
	local isLink, playerName = IsTradeSkillLinked()
	if not isLink then
		-- We are looking at our own tradeskill
		playerName = UnitName("player")
	end
	
	--crafterDB[playerName] = tradeLink
	--self:print(playerName .. ": " .. tradeLink)
	
	for i=1,GetNumTradeSkills() do
		local skillName, skillType, _, _, _ = GetTradeSkillInfo(i)
		if skillName and skillType ~= "header" then
			local spellLink = GetTradeSkillRecipeLink(i)
			-- |cffffd000|Henchant:9060|h[Leatherworking: Light Leather Quiver]|h|r
			local spellID = gsub(spellLink, "^|c%x+|H%l+:(%d+)|.*", "%1")
			spellID = tonumber(spellID)
			
			--self:print(spellID .. " " .. strbyte(spellID, 2) .. " " .. strchar(strbyte(spellID, 2)))
			if not db[tradeID][spellID] then
				db[tradeID][spellID] = {}
			end
			if self:InTable(playerName, db[tradeID][spellID]) == nil then
				db[tradeID][spellID][#db[tradeID][spellID]+1] = playerName
			end
			
			--self:print(spellID .. " " .. spellLink .. gsub(spellLink, "\124", "\124\124"))
			local spellName = GetSpellInfo(spellID)
			if not spellDB[tradeID][spellID] then
				--self:print("Adding spell " .. spellName .. " (" .. spellID .. ")")
				spellDB[tradeID][spellID] = spellName
			end
		end
	end
	self:SaveSV()
end

function CraftDB:InTable(needle, haystack)
	for k,v in pairs(haystack) do
		if(v == needle) then
			return k
		end
	end
	return nil
end

-- Load DB
function CraftDB:ADDON_LOADED(self, addon)
	if addon == "CraftDB" then
		if not CraftDBSV then CraftDBSV = {} end
		
		if not CraftDBSV[serverfaction] then
			CraftDBSV[serverfaction] = {}
			for _, id in pairs(tradeIDs) do
				CraftDBSV[serverfaction][id] = {}
			end
		end
		db = CraftDBSV[serverfaction]
		
		if not CraftDBSV["spells"] then
			CraftDBSV["spells"] = {}
			for _, id in pairs(tradeIDs) do
				CraftDBSV["spells"][id] = {}
			end
		end
		spellDB = CraftDBSV["spells"]

		--self:print("Initialized DB")
		
		SLASH_CRAFTDB1 = "/craftdb"
		SLASH_CRAFTDB2 = "/cdb"
		SlashCmdList["CRAFTDB"] = function (arg) self:SlashCmd(arg) end
		
		if DataBroker then
			local launcher = DataBroker:NewDataObject("Craft DB", {
				type = "launcher",
				icon = "Interface\\Icons\\INV_Misc_Book_06",
				OnClick = function(clickedframe, button)
					--LibStub("AceConfigDialog-3.0"):Open("ClassTimer")
					if self:IsVisible() then
						self:Hide()
					else
						self:Show()
					end
				end,
			})	
		end
		
		self:CreateWindow()
	end
end

-- Save DB
function CraftDB:PLAYER_LOGOUT(self)
	self:SaveSV()
end

function CraftDB:SaveSV()
	--CraftDBSV = nil
	CraftDBSV[serverfaction] = db
	CraftDBSV["spells"] = spellDB
end

function CraftDB:SlashCmd(arg)
	if self:IsVisible() then
		self:Hide()
	else
		self:Show()
	end
	--[[
	-- assum arg is an spellID
	if not arg or arg == "" then
		self:Show()
		return
	end
	
	arg = tonumber(arg)
	for _,sp in pairs(db) do
		--self:print("Searching " .. GetSpellLink(id) .. " for " .. GetSpellLink(arg) .. " (" .. arg .. ")")
		for id,names in pairs(sp) do
			id = tonumber(id)
			if id == arg then
				local p = ""
				for _,name in pairs(names) do
					if p == "" then
						p = name
					else
						p = p .. ", " .. name
					end
				end
				self:print(GetSpellLink(id) .. ": " .. p)
				return
			end
		end
	end
	]]
end

function CraftDB:CreateWindow()
	-- Make the frame closeable with esc
	tinsert(UISpecialFrames, self:GetName());

	self:SetFrameStrata("DIALOG")
	self:SetMovable(true)
	self:EnableMouse(true)
	self:SetScript("OnMouseDown", function() self:StartMoving() end)
	self:SetScript("OnMouseUp", function() self:StopMovingOrSizing() end)
	self:SetScript("OnShow", function() PlaySound("igCharacterInfoOpen") end)
	self:SetScript("OnHide", function() PlaySound("igCharacterInfoClose") end)
	self:SetWidth(700)
	self:SetHeight(500)
	self:SetPoint("CENTER", UIParent, "CENTER")
	self:SetBackdrop({
		bgFile = "Interface/DialogFrame/UI-DialogBox-Background",
		edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true,
		tileSize = 32,
		edgeSize= 32,
		insets = {left=11, right=12, top=12, bottom=11},
	})
	
	self.FrameHeader = self:CreateTexture("$parent_FrameHeader", "ARTWORK")
	self.FrameHeader:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Header")
	self.FrameHeader:SetWidth(200)
	self.FrameHeader:SetHeight(64)
	self.FrameHeader:SetPoint("TOP", 0, 12)
	
	self.FrameTitle = self:CreateFontString("$parten_FrameTitle", "ARTWORK", "GameFontNormal")
	self.FrameTitle:SetText("Craft DB")
	self.FrameTitle:SetPoint("TOP", 0, -2)
	
	self.CloseBtn = CreateFrame("Button", "$parent_CloseBtn", self, "UIPanelButtonTemplate")
	self.CloseBtn:SetWidth(80)
	self.CloseBtn:SetHeight(22)
	self.CloseBtn:SetPoint("BOTTOMRIGHT", -11, 12)
	self.CloseBtn:SetText("Close")
	self.CloseBtn:SetScript("OnClick", function() self:Hide() end)
	
	self.TradeDropDown = CreateFrame("Frame", "$parent_TradeDropDown", self, "UIDropDownMenuTemplate")
	self.TradeDropDown:SetID(1)
	self.TradeDropDown:SetPoint("TOPLEFT", 0, -40)
	--self.TradeDropDown:SetScript("OnLoad", self.TradeDropDown_Initialize)
	self.TradeDropDown:SetScript("OnShow", function()
		UIDropDownMenu_SetWidth(CraftDB.TradeDropDown, 120);
		UIDropDownMenu_Initialize(CraftDB.TradeDropDown, self.TradeDropDown_Initialize);
		UIDropDownMenu_SetSelectedID(CraftDB.TradeDropDown, 1);
		UIDropDownMenu_JustifyText(CraftDB.TradeDropDown, "LEFT");
	end)
	
	self.TradeDropDown.Title = self.TradeDropDown:CreateFontString("$parent_Text", "ARTWORK", "GameFontHighlightSmall")
	self.TradeDropDown.Title:SetJustifyH("LEFT")
	self.TradeDropDown.Title:SetPoint("BOTTOMLEFT", self.TradeDropDown, "TOPLEFT", 20, 0)
	self.TradeDropDown.Title:SetText("Profession")
	
	self.FrameTitle = self:CreateFontString("$parten_FrameTitle", "ARTWORK", "GameFontNormal")
	self.FrameTitle:SetText("Craft DB")
	self.FrameTitle:SetPoint("TOP", 0, -2)
	
	self.SearchBox = CreateFrame("EditBox", "$parent_SearchBox", self, "InputBoxTemplate")
	self.SearchBox:SetWidth(520)
	self.SearchBox:SetHeight(16)
	self.SearchBox:SetAutoFocus(false)
	self.SearchBox:SetPoint("TOPLEFT", self.TradeDropDown, "TOPRIGHT", -7, -6)
	self.SearchBox:SetScript("OnEscapePressed", function() self.SearchBox:ClearFocus() end)
	self.SearchBox:SetScript("OnTabPressed", function() self.SearchBox:ClearFocus() end)
	self.SearchBox:SetScript("OnEnterPressed", function() self.SearchBox:ClearFocus() end)
	self.SearchBox:SetScript("OnTextChanged", self.SearchBoxChange)
	
	self.SearchBox.Title = self.SearchBox:CreateFontString("$parent_Text", "ARTWORK", "GameFontHighlightSmall")
	self.SearchBox.Title:SetJustifyH("LEFT")
	self.SearchBox.Title:SetPoint("BOTTOMLEFT", self.SearchBox, "TOPLEFT", 0, 4)
	self.SearchBox.Title:SetText("Search")
	
	-- Border for the spell list
	self.Border = CreateFrame("Frame", "$parent_Border", self)
	self.Border:SetWidth(670)
	self.Border:SetHeight(390)
	self.Border:SetPoint("TOPLEFT", 15, -70)
	self.Border:SetBackdrop({
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile = false,
		tileSize = 16,
		edgeSize= 16,
		insets = {left=5, right=5, top=5, bottom=5},	
	})
	self.Border.Texture = self.Border:CreateTexture("$parent_Texture", "BACKGROUND")
	self.Border.Texture:SetWidth(666)
	self.Border.Texture:SetHeight(386)
	self.Border.Texture:SetPoint("CENTER")
	self.Border.Texture:SetTexture(0.1, 0.1, 0.1, 1)

	-- Lets create some Buttons
	self.Spell1 = CreateFrame("Button", "$parent_Spell1", self)
	self.Spell1:SetWidth(280)
	self.Spell1:SetHeight(18)
	self.Spell1:SetPoint("TOPLEFT", self.Border.Texture, "TOPLEFT", 5, -5)
	self.Spell1:SetNormalFontObject("GameFontHighlightLeft")
	self.Spell1:SetScript("OnClick", self.SpellButtonClick)
	self.Spell1:SetScript("OnEnter", self.SpellButtonEnter)
	self.Spell1:SetScript("OnLeave", function() _G["GameTooltip"]:Hide() end)
	--self.Spell1:SetText("TestSpell1")

	self.Names1 = CreateFrame("Button", "$parent_Names1", self)
	self.Names1:SetWidth(350)
	self.Names1:SetHeight(18)
	self.Names1:SetPoint("TOPLEFT", self.Spell1, "TOPRIGHT")
	self.Names1:SetNormalFontObject("GameFontHighlightLeft")
	self.Names1:SetScript("OnEnter", self.NamesButtonEnter)
	self.Names1:SetScript("OnLeave", function() _G["GameTooltip"]:Hide() end)
	--self.Names1:SetText("TestName1")

	for i = 2, 21, 1 do
		self["Spell"..i] = CreateFrame("Button", "$parent_Spell"..i, self)
		self["Spell"..i]:SetWidth(280)
		self["Spell"..i]:SetHeight(18)
		self["Spell"..i]:SetPoint("TOPLEFT", self["Spell"..i-1], "BOTTOMLEFT")
		self["Spell"..i]:SetNormalFontObject("GameFontHighlightLeft")
		self["Spell"..i]:SetScript("OnClick", self.SpellButtonClick)
		self["Spell"..i]:SetScript("OnEnter", self.SpellButtonEnter)
		self["Spell"..i]:SetScript("OnLeave", function() _G["GameTooltip"]:Hide() end)
		
		self["Names"..i] = CreateFrame("Button", "$parent_Names"..i, self)
		self["Names"..i]:SetWidth(350)
		self["Names"..i]:SetHeight(18)
		self["Names"..i]:SetPoint("TOPLEFT", self["Spell"..i], "TOPRIGHT")
		self["Names"..i]:SetNormalFontObject("GameFontHighlightLeft")
		--self["Names"..i]:SetScript("OnEnter", self.NamesButtonEnter)
		self["Names"..i]:SetScript("OnEnter", self.NamesButtonEnter)
		self["Names"..i]:SetScript("OnLeave", function() _G["GameTooltip"]:Hide() end)
		--self["Spell"..i]:SetText("TestSpell"..i)
	end

	self.ScrollFrame = CreateFrame("ScrollFrame", "$parent_ScrollFrame", self, "FauxScrollFrameTemplate")
	self.ScrollFrame:SetPoint("TOPLEFT", self.Border.Texture, "TOPLEFT", 0, -2)
	self.ScrollFrame:SetPoint("BOTTOMRIGHT", self.Border.Texture, "BOTTOMRIGHT", -24, 2)
	self.ScrollFrame:SetScript("OnShow", self.ScrollFrameUpdate)
	--self.ScrollFrame:SetScript("OnVerticalScroll", self.ScrollFrameScroll)
	self.ScrollFrame:SetScript("OnVerticalScroll", function(self, offset) FauxScrollFrame_OnVerticalScroll(self, offset, 16, CraftDB.ScrollFrameUpdate) end)

	self:Hide()
end

function CraftDB:SearchBoxChange()
	--CraftDB:print(self:GetText())
	if self:GetText() ~= "" then
		display = {}
		
		local text = strlower(self:GetText())
		for _, v in pairs(spellDB[displayTrade]) do
			local find = strfind(strlower(v), text)
			if find then
				display[#display + 1] = v
			end
		end
		
		sort(display)
	else
		CraftDB:UpdateListFromTrade()
	end
	CraftDB.ScrollFrame:Hide()
	CraftDB.ScrollFrame:Show()
end

function CraftDB:TradeDropDown_Initialize()
	--CraftDB:print("TradeDropDown Initialize")
	
	local info = UIDropDownMenu_CreateInfo()
	for _, id in pairs(tradeIDs) do
		--CraftDB:print(id .. " " .. GetSpellInfo(id))
	 	info.text = GetSpellInfo(id)
	 	info.value = id
		info.func = CraftDB.TradeDropDown_OnClick
		info.checked = nil
		UIDropDownMenu_AddButton(info);
	end
	displayTrade = tradeIDs[1]
	CraftDB:UpdateListFromTrade()
end

-- This function is part of a table cotaining the dropdownmenu entries, so we can use this, this:GetID, etc
function CraftDB:TradeDropDown_OnClick()
	--CraftDB:print(this:GetID() .. " " .. tradeIDs[self:GetID()] .. " " .. GetSpellLink(tradeIDs[self:GetID()]))
	UIDropDownMenu_SetSelectedID(CraftDB.TradeDropDown, self:GetID());
	
	-- Update spell list
	if displayTrade ~= tradeIDs[self:GetID()] then
		displayTrade = tradeIDs[self:GetID()]
		CraftDB:UpdateListFromTrade()
	end
end

function CraftDB:UpdateListFromTrade()
	display = {}
	local i = 1
	--self:print(displayTrade)
	
	for _, a in pairs(spellDB[displayTrade]) do
		display[i] = a
		i = i + 1
	end
	sort(display)
	--self:SortDisplayList()
	--for _, v in pairs(display) do
	--	self:print(v)
	--end
	self.ScrollFrame:Hide()
	self.ScrollFrame:Show()
end

function CraftDB:ScrollFrameUpdate()	
	local line; -- 1 through 5 of our window to scroll
	local lineplusoffset; -- an index into our data calculated from the scroll offset
	FauxScrollFrame_Update(self, #display, 21, 16);
	
	for line=1, 21 do
		lineplusoffset = line + FauxScrollFrame_GetOffset(self);
		if lineplusoffset <= #display then
			CraftDB["Spell"..line]:SetText(display[lineplusoffset]);
			
			-- Intelligent Displaying of names ^.^
			local text = CraftDB:FormatNames(db[displayTrade][CraftDB:InTable(display[lineplusoffset], spellDB[displayTrade])])
			
			CraftDB["Names"..line]:SetText(text)
			CraftDB:CropButtonText(CraftDB['Spell'..line])
			CraftDB:CropButtonText(CraftDB['Names'..line])
			CraftDB["Spell"..line]:Show()
			CraftDB["Names"..line]:Show()
		else
			CraftDB["Spell"..line]:Hide()
			CraftDB["Names"..line]:Hide()
		end
	end
end

function CraftDB:FormatNames(n)
	--self:print(table.concat(n, ", "))
	
	local names = {}
	for k,v in pairs(n) do
		names[k] = v
	end
	local text = ""

	local guildOnline = {}
	local guildOffline = {}
	
	for i = 1,GetNumGuildMembers(true) do
		local name, _, _, _, _, _, _, _, online = GetGuildRosterInfo(i)
		if online then
			guildOnline[#guildOnline + 1] = name
		else
			guildOffline[#guildOffline + 1] = name
		end
	end
	
	local friendsOnline = {}
	local friendsOffline = {}
	
	for i = 1,GetNumFriends() do
		local name, _, _, _, online = GetFriendInfo(i)
		--self:print(name)
		if online then
			friendsOnline[#friendsOnline + 1] = name
		else
			friendsOffline[#friendsOffline + 1] = name
		end
	end

	-- Online guild members in green			
	local namestmp = {}
	for k, n in pairs(names) do
		--CraftDB:print("Checking if " .. n .. " is online in Guild")
		if CraftDB:InTable(n, guildOnline) then
			--CraftDB:print(n .. " is in Guild online")
			namestmp[#namestmp + 1] = n
			names[k] = nil
		end
	end
	if #namestmp > 0 then
		sort(namestmp)
		text = text .. "|cff00ff00" .. table.concat(namestmp, "|r, |cff00ff00") .. "|r"
	end
	
	-- Offline guild members in dark green
	local namestmp = {}
	for k, n in pairs(names) do
		--CraftDB:print("Checking if " .. n .. " is online in Guild")
		if CraftDB:InTable(n, guildOffline) then
			--CraftDB:print(n .. " is in Guild online")
			namestmp[#namestmp + 1] = n
			names[k] = nil
		end
	end
	if #namestmp > 0 then
		sort(namestmp)
		if text ~= "" then
			text = text .. ", "
		end
		text = text .. "|cff40c040" .. table.concat(namestmp, "|r, |cff40c040") .. "|r"
	end

	-- Online friends in bright yellow
	--self:print(table.concat(friendsOnline, ", "))
	local namestmp = {}
	for k, n in pairs(names) do
		--CraftDB:print("Checking if " .. n .. " is online in Guild")
		if CraftDB:InTable(n, friendsOnline) then
			--CraftDB:print(n .. " is in Guild online")
			namestmp[#namestmp + 1] = n
			names[k] = nil
		end
	end
	if #namestmp > 0 then
		sort(namestmp)
		if text ~= "" then
			text = text .. ", "
		end
		text = text .. "|cffffff00" .. table.concat(namestmp, "|r, |cffffff00") .. "|r"
	end
	
	-- Offline friends in dark yellow
	--self:print(table.concat(friendsOffline, ", "))
	local namestmp = {}
	for k, n in pairs(names) do
		--CraftDB:print("Checking if " .. n .. " is online in Guild")
		if CraftDB:InTable(n, friendsOffline) then
			--CraftDB:print(n .. " is in Guild online")
			namestmp[#namestmp + 1] = n
			names[k] = nil
		end
	end
	if #namestmp > 0 then
		sort(namestmp)
		if text ~= "" then
			text = text .. ", "
		end
		text = text .. "|cffeac117" .. table.concat(namestmp, "|r, |cffeac117") .. "|r"
	end

	-- All the other names
	namestmp = {};
	
	-- restor names w/o nil values
	for _, n in pairs(names) do
		if n ~= nil then
			namestmp[#namestmp + 1] = n
		end
	end
	if #namestmp > 0 then
		sort(namestmp)
		if text ~= "" then
			text = text .. ", "
		end
		text = text .. table.concat(namestmp, ", ")
	end
	
	
	--[[
	sort(namestmp)
	for _, n in pairs(namestmp) do
		if n then
			if text ~= "" then
				text = text .. ", "
			end
			text = text .. n
		end
	end
	]]
	--self:print(text)
	return text
end

function CraftDB:CropButtonText(button)
	local width = button:GetWidth()
--	local textWidth = buton:GetTextWidth()
	button.fullText = button:GetText()
	local text = button:GetText()
	--self:print(width .. " " .. button:GetTextWidth())
	while button:GetTextWidth() > width do
		text = strsub(text, 1, strlen(text) - 1)
		button:SetText(text.."...")
		--button:SetWidth(button:GetTextWidth()) -- No effect
	end
end

function CraftDB:SpellButtonEnter()
	spellID = CraftDB:InTable(self.fullText, spellDB[displayTrade])
	if not spellID then return end
	
	--CraftDB:print(self:GetText() .. " " ..spellID)
	
	_G["GameTooltip"]:SetOwner(CraftDB, "ANCHOR_CURSOR")
	_G["GameTooltip"]:SetHyperlink("enchant:" .. spellID)
	--_G["GameTooltip"]:Show()
end

function CraftDB:NamesButtonEnter()
	--CraftDB:print("Doing shit")
	
	_G["GameTooltip"]:SetOwner(CraftDB, "ANCHOR_CURSOR")
	_G["GameTooltip"]:SetText(self.fullText, 1, 1, 1, 1, 1)
	--_G["GameTooltip"]:Show()
end

function CraftDB:SpellButtonClick()
	spellID = CraftDB:InTable(self.fullText, spellDB[displayTrade])
	if not spellID then return end
	
	--CraftDB:print(self:GetText() .. " " ..spellID)
	
	_G["ChatFrame1EditBox"]:SetText(_G["ChatFrame1EditBox"]:GetText() .. " " .. GetSpellLink(spellID) .. " ")
end

function CraftDB:GetRealTradeID(id)
	local ts = {
		-- Alchemy
		[2259]	= 2259,
		[3101]	= 2259,
		[3464]	= 2259,
		[11611]	= 2259,
		[28596]	= 2259,
		[28672] = 2259,
		[28675]	= 2259,
		[28677]	= 2259,
		[51304]	= 2259,
        [80731] = 2259,
        [105206] = 2259,
		-- Blacksmithing
        [2018] = 2018,
		[3100] = 2018,
		[3538] = 2018,
		[9785] = 2018,
		[29844]	= 2018,
		[51300]	= 2018,
        [76666] = 2018,
        [110396] = 2018,
		-- Enchanting
		[7411]	= 7411,
		[7412]	= 7411,
		[7413]	= 7411,
		[13920]	= 7411,
		[28029]	= 7411,
		[51313]	= 7411,
        [74258] = 7411,
        [110400] = 7411,
		-- Engineering
		[4036]	= 4036,
		[4037]	= 4036,
		[4038]	= 4036,
		[12656]	= 4036,
		[20219]	= 4036,
		[20222]	= 4036,
		[30350]	= 4036,
		[51306]	= 4036,
        [82774] = 4036,
        [110403] = 4036,
		-- Inscription
		[45357]	= 45357,
		[45358]	= 45357,
		[45359]	= 45357,
		[45360]	= 45357,
		[45361]	= 45357,
		[45363]	= 45357,
        [86008] = 45357,
        [110417] = 45357,
		-- Jewelcrafting
		[25229]	= 25229,
		[25230]	= 25229,
		[28894]	= 25229,
		[28895]	= 25229,
		[28897]	= 25229,
		[51311]	= 25229,
        [73318] = 25229,
        [110420] = 25229,
		-- Leatherworking
		[2108]	= 2108,
		[3104]	= 2108,
		[3811]	= 2108,
		[10662]	= 2108,
		[32549]	= 2108,
		[51302]	= 2108,
        [81199] = 2108,
        [110423] = 2108,
		-- Tailoring
		[3908]	= 3908,
		[3909]	= 3908,
		[3910]	= 3908,
		[12180]	= 3908,
		[26790]	= 3908,
		[51309]	= 3908,
        [75156] = 3908,
        [110426] = 3908,
		-- Cooking
		[2550]	= 2550,
		[3102]	= 2550,
		[3413]	= 2550,
		[18260]	= 2550,
		[33359]	= 2550,
		[51296]	= 2550,
        [88053] = 2550,
        [104381] = 2550,
		-- First Aid
		[3273] = 3273,
		[3274] = 3273,
		[7924] = 3273,
		[10846]	= 3273,
		[27028]	= 3273,
		[45542]	= 3273,
        [74559] = 3273,
        [110406] = 3273,
	}
	if type(id) == "string" then id = tonumber(id) end
	if not ts[id] then
		self:print("TradeID not found:" .. id)
		return nil	
	end
	return ts[id]
end

function CraftDB:print(str)
	if not str then return end
	DEFAULT_CHAT_FRAME:AddMessage(format("|cff00ffffCraftDB: |r %s", str))
end

